#include "readplate.h"
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

int tot_plates;     /* Define some global variables - messy I know */
int valid_plates;  

PLATE *read_plate_list (char *fname)
{
    FILE *fptr;
    char line[MAXLEN];
    PLATE *list;
    PLATE *tmpptr;
    PLATE *newplate;
    
    tot_plates= 0;
    valid_plates= 0;
    fptr= fopen (fname, "r");
    if (fptr == NULL) 
        return NULL;
    tmpptr= NULL;
    list= NULL;
    
    while (fgets (line, MAXLEN - 1, fptr) != NULL) {
        newplate= read_plate_line (line);
	if (newplate == NULL)
	    continue;
	if (list == NULL) {
	    list= newplate;
	} else {
	    tmpptr->next= newplate;
	}

	tmpptr= newplate;
	tmpptr->next= NULL;
    }
    fclose (fptr);
    frig_times(list);
    frig_plates(list);
    return list;
}

PLATE *read_plate_line (char *line)
{
 
    PLATE *newplate;
    int i= 0;
    int j,k;
    char word[MAXLEN];
    
    newplate= malloc (sizeof (PLATE));
    newplate->time= NO_TIME;
    newplate->plate[0]= '\0';
    newplate->info[0]='\0';
    
    while ( (j= read_next_part(line, word, i)) != END_OF_LINE) {
        switch (parse_word(word)) {
	    case (IS_PLATE) :
	        
	        for (k= 0; k < strlen(word); k++) {
		    if (isalpha((int)word[k]))
		        word[k]= (char)toupper((int)word[k]);
		}     
		strncpy (newplate->plate, word, PLATE_LEN);
		break;
	    case (IS_TIME) :
	        newplate->time= convtime(word);
	        break;
            case (IS_COMMENT) : 
	        if (isspace((int)line[i])) 
		    i++;
	        if (line[i] == '\n' || line[i] == '\r' || line[i] == '\0') {
		    j= END_OF_LINE;
		    break;
		}
		if (newplate->plate[0] == '\0') {
    		    if (line[i] == '#') { 
	                strncpy (newplate->info, &line[i], COMMENT_LEN); 
		    } else {
		         newplate->info[0]='#';
		        strncpy (&newplate->info[1], &line[i], COMMENT_LEN-1);
		    }
		    j= strlen(newplate->info)-1;
		    while (newplate->info[j] == '\r' || newplate->info[j] == '\n') {
		        newplate->info[j]= '\0';
		        j--;
		    }	
	            j= END_OF_LINE;	
		} else {
		    if (strlen (newplate->info) > 0 && strlen(newplate->info) < COMMENT_LEN-1)
		       strcat (newplate->info," ");
		    strncat (newplate->info, word, COMMENT_LEN-1-strlen(newplate->info));
		}
		
		break;
	    case (IS_INFO_LINE) :
	        parse_info_line(line);
		j= END_OF_LINE;
		break;
	}
	if (j == END_OF_LINE) 
	    break;
	i= j;
    }
    if (newplate->plate[0] == '\0' && newplate->info[0] == '\0')
        return NULL;
    return newplate;
}

int read_next_part (char line[MAXLEN], char word[MAXLEN], int i)
{
    int j= 0;
    while (isspace((int)line[i]))
        i++;
    while (!isspace((int)line[i]) && line[i] != '\n' && line[i] != '\r' && 
        line[i] != '\0') {
	word[j]= line[i];
	j++;
	i++;
    }
    word[j]= '\0';
    if (isspace((int)line[i]))
        return i;
    return END_OF_LINE;
}

int parse_word (char word[MAXLEN])
{
    int i;
    int no_digits;
    int no_punct;
    
    no_digits= 0;
    no_punct= 0;
    if (strcmp (word, "#INFO") == 0)
        return IS_INFO_LINE;
    for (i= 0; i < strlen(word); i++) {
        if (isdigit((int) word[i]))
	    no_digits++;
	if (ispunct((int) word[i]))
	    no_punct++;
    }
    if (no_punct == 1 && no_digits > 3 && no_digits + no_punct == strlen(word))
        return IS_TIME;
    no_digits= 0;
    no_punct= 0;
    for (i= 0; i < strlen(word); i++) {
        if (isdigit((int) word[i]))
	    no_digits++;
	if (ispunct((int) word[i]))
	    no_punct++;
    }
    if (no_digits > 0 && no_punct == 0)
        return IS_PLATE;
    return IS_COMMENT;
}

void parse_info_line (char line[MAXLEN])
{
    char word [MAXLEN];
    int i= 0;
    int j;
    j= read_next_part(line, word, i);
    
    if (j == END_OF_LINE) {
        fprintf (stderr, "Error in info line!\n");
	exit (-1);
    }
    i= j;
    j= read_next_part(line, word, i);
    if (j == END_OF_LINE) {
        fprintf (stderr, "Error in info line!\n");
	exit (-1);
    }
    tot_plates= atoi (word);
    if (tot_plates <= 0) {
        fprintf (stderr, "Error in info line!\n");
	exit (-1);
    }
}

int convtime(char word[MAXLEN])

{
   int hrs= 0;
   int mins= 0;
   int i= 0;
   hrs= atoi (word);
   while (! ispunct( (int) word[i]))
       i++;
   i++;
   mins= atoi (&word[i]);
   return (hrs*60+mins);
}

void hrsmins (int time, int *hrs, int *mins)

{
    *hrs= (time/60);
    *mins= time - ((*hrs)*60);
}

void frig_times (PLATE *list)

{
    PLATE *lasttime;
    PLATE *tmpptr;
    int no_plates= 0;
    int firsttime;
    float realtime;
    int hrs1, mins1;
    int hrs2, mins2;

    lasttime= NULL;
    tmpptr= list;
    while (tmpptr != NULL) {
        if (tmpptr->plate[0] == '\0') {
	    tmpptr= tmpptr->next;
	    continue;
	} 
        if (tmpptr->time == NO_TIME && tmpptr->next != NULL) {
	    no_plates++;
	    tmpptr= tmpptr->next;
	    continue;
	}
	
        if (lasttime == NULL) {
	    firsttime= tmpptr->time;
	    lasttime= list;
        } else {
	    firsttime= lasttime->time;
	}
	if (tmpptr->time == NO_TIME) 
	    tmpptr->time= firsttime;
	if (firsttime == NO_TIME) {
	    fprintf (stderr, "No times given\n");
	    exit(-1);
	}
	if (tmpptr->time < firsttime) {
	    hrsmins(tmpptr->time, &hrs1, &mins1);
	    hrsmins(firsttime, &hrs2, &mins2);
	    fprintf (stderr, "Time reversal between %d:%02d and %d:%02d\n",
		hrs1, mins1, hrs2, mins2);
	    exit (-1);
	}
	if (no_plates > 0) {
            realtime= (float) (tmpptr->time - firsttime) / (float) (no_plates+1);
	    if (lasttime->time != NO_TIME) 
		lasttime= lasttime->next;
	    while (lasttime != tmpptr) {
	        if (lasttime->plate[0] != '\0') {
		    lasttime->time= firsttime+ (int)realtime;
		    realtime+= (float) (tmpptr->time - firsttime) /
			 (float) (no_plates+1);
		}
		lasttime= lasttime->next;
	    }
	    no_plates= 0;
	}
	lasttime= tmpptr;
        tmpptr= tmpptr->next;
    }   
}

void frig_plates (PLATE *list)
{
    int no_plates= 0;
    PLATE *tmpptr;
    int i,j;
    char first_lett[PLATE_LEN];   /* First group of letters in plate */
    char first_num[PLATE_LEN];    /* First group of numbers in plate */
    char second_lett[PLATE_LEN];  /* Second group of letters in plate */
    char rest_num[PLATE_LEN];     /* Rest of numbers in plate */
    for (tmpptr= list; tmpptr != NULL; tmpptr= tmpptr->next) {
        first_lett[0]='\0';
	first_num[0]= '\0';
	second_lett[0]= '\0';
	rest_num[0]= '\0';
	for (i= 0; i < strlen(tmpptr->plate); i++) {
	    if (isalpha((int)tmpptr->plate[i]))
	        first_lett[i]= tmpptr->plate[i];
	    else
	        break;
	}
	first_lett[i]= '\0';
	j= 0;
	for (; i < strlen(tmpptr->plate); i++) {
           if (isdigit((int)tmpptr->plate[i])) {
	       first_num[j]= tmpptr->plate[i];
	       j++;
	   } else { 
	       break;
	   }
        }
	first_num[j]='\0';
	j= 0;
	for (; i < strlen(tmpptr->plate); i++) {
           if (isalpha((int)tmpptr->plate[i])) {
	       second_lett[j]= tmpptr->plate[i];
	       j++;
	   } else { 
	       break;
	   }
        }
	second_lett[j]='\0';
	j= 0;
	for (; i < strlen(tmpptr->plate); i++) {
           if (isdigit((int)tmpptr->plate[i])) {
	       rest_num[j]= tmpptr->plate[i];
	       j++;
	   } 
        }
	rest_num[j]='\0';
	/* If the format has two separate number groups then it is not a
	standard english plate - take only the numbers */
	if (strlen(rest_num) > 0) {
	    sprintf (tmpptr->plate, "%s%s",first_num, rest_num);
	    continue;
	}
	/* If we have a letter some numbers and possibly a second letter group
	then this is a standard english plate e.g. A 124 SGD*/
	if (strlen(first_lett) == 1) {
	    sprintf (tmpptr->plate, "%s%s",first_lett, first_num);
	    continue;
	}
	/* If we have possibly some letters, some numbers and a single letter
	then this is a standard english plate e.g. SDG 124 A */
	if (strlen (second_lett) == 1) {
	    sprintf (tmpptr->plate, "%s%s",first_num, second_lett);
	    continue;
	}
	/* We have some other non standard plate - take only the numbers */
	sprintf (tmpptr->plate, "%s%s",first_num, rest_num);
    }
    tmpptr= list;
    while (tmpptr != NULL) {
        if (strlen (tmpptr->plate) > 0 && strlen (tmpptr->plate) < MIN_PLATE) {
	    sprintf (tmpptr->info,"#%s",tmpptr->plate);
	    tmpptr->plate[0]= '\0';
	}
        tmpptr= tmpptr->next;
    }
    for (tmpptr= list; tmpptr != NULL; tmpptr= tmpptr->next) {
        if (tmpptr->plate[0] != '\0') {
	    no_plates++;
	    valid_plates++;
	} else if (strlen(tmpptr->info) < PLATE_LEN){
	    no_plates++;
	}
    }
    if (tot_plates == 0)
        tot_plates= no_plates;
}

void write_plate_list (FILE *fptr, PLATE *list)
{
    PLATE *tmpptr;
    int hrs, mins;
    tmpptr= list;
    
    fprintf (fptr,"#INFO %d\n",tot_plates);
    while (tmpptr != NULL) {
        if (tmpptr->plate[0] == '\0') {
	    fprintf (fptr,"%s\n",tmpptr->info);
	} else {
	    hrsmins(tmpptr->time, &hrs, &mins);
	    fprintf (fptr,"%-*s %02d:%02d     %s\n",PLATE_LEN, tmpptr->plate, hrs, mins, 
	        tmpptr->info);
	}    
        tmpptr= tmpptr->next;
    }
}

void free_plate_list (PLATE *list)
{
    PLATE *tmpptr;
    while (list != NULL) {
        tmpptr= list;
	list= list->next;
	free (tmpptr);
    }

}
