#ifndef _READPLATE_H_
#define _READPLATE_H_
#include <stdio.h>

enum {MAXLEN= 256, 	/* Max length of lines read from file */
      COMMENT_LEN= 60,  /* Max length of comment */
      PLATE_LEN= 10,    /* Max length of licence plate */
      NO_TIME= -1,      /* Line has no time stamp */
      END_OF_LINE= -1,	/* End of line in parsing a plate line */
      IS_PLATE= 1,      /* Word read in line is number plate */
      IS_TIME= 2,	/* Word read in line is time of day */
      IS_COMMENT= 3,	/* Word read in line is comment */
      IS_INFO_LINE= 4,  /* Word read in line is info line */
      MIN_PLATE=3     /* Minimum plate length which will allow sensible match*/
      };

extern int tot_plates;  /* Number of plates that there are in total */
extern int valid_plates;  /* Number of valid plates */

typedef struct _plate_ {
    int time;
    char info [COMMENT_LEN];
    char plate [PLATE_LEN];
    struct _plate_ *next;
} PLATE;

PLATE *read_plate_list (char *fname);
/* Return a linked list of plates */

PLATE *read_plate_line (char *line);
/* Return a single plate line */

void free_plate_list (PLATE *list);
/* Free a list of plates */


int read_next_part (char line[MAXLEN], char word[MAXLEN], int i);
/* Read next part of line into word */

int parse_word (char word[MAXLEN]);
/* Parse word to determine if it is a plate or a comment or a time */

void parse_info_line(char line[MAXLEN]);
/* Parse special info line containing info about file */

int convtime(char word[MAXLEN]);
/* Convert a time as 9:15 into a minutes past midnight time */

void frig_times(PLATE *);
/* Change times on plates to interpolate missing times */

void frig_plates (PLATE *list);
/* Remove short plates which will match too easily */

void write_plate_list (FILE *fptr,PLATE *list);
/* Write a list of plates prettily formatted */



#endif
